# Python Software Foundation License
from odict import odict
from odict.pyodict import _nil
from odict.pyodict import _odict
import copy
import pickle
import sys


try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestOdict(unittest.TestCase):

    def assertRaisesWithMessage(self, msg, func, exc, *args, **kwargs):
        try:
            func(*args, **kwargs)
        except exc as inst:
            self.assertEqual(str(inst), msg)

    def test_abstract_superclass(self):
        # Abstract superclass provides no concrete _dict_impl class.
        msg = 'No dict implementation class provided.'
        self.assertRaisesWithMessage(msg, _odict, TypeError)
        o = odict()
        self.assertEqual(o._dict_impl(), dict)
        del o

    def test_init_with_kw(self):
        # Initialization with keyword arguments fails.
        msg = (
            '__init__() of ordered dict takes no keyword arguments to avoid '
            'an ordering trap.'
        )
        self.assertRaisesWithMessage(msg, odict, TypeError, a=1)

    def test_init_with_dict(self):
        # When initialized with a dict instance, the order of elements is
        # undefined.
        o = odict({'b': 2, 'a': 1, 'd': 4, 'c': 3})
        self.assertEqual(sorted(o.keys()), ['a', 'b', 'c', 'd'])
        self.assertEqual(o['a'], 1)
        self.assertEqual(o['b'], 2)
        self.assertEqual(o['c'], 3)
        self.assertEqual(o['d'], 4)

    def test_init_with_items(self):
        # When initialized with a list instance, the order of elements is
        # preserved.
        o = odict([('a', 1), ('b', 2), ('c', 3), ('d', 4)])
        self.assertEqual(o.items(), [('a', 1), ('b', 2), ('c', 3), ('d', 4)])

    def test_containment(self):
        # Test key containment with __iter__() and has_key().
        o = odict([('a', 1)])
        self.assertTrue('a' in o)
        self.assertFalse('foo' in o)
        self.assertTrue(o.has_key('a'))

    def test_get(self):
        # Test fetching values with get()
        o = odict([('a', 1)])
        self.assertEqual(o.get('a'), 1)
        self.assertEqual(o.get('foo', ''), '')

    def test_values(self):
        # Test fetching values with values()
        o = odict([('a', 1), ('b', 2), ('c', 3), ('d', 4)])
        self.assertEqual(o.values(), [1, 2, 3, 4])

    def test_update_with_kw(self):
        # update() with keyword arguments fails.
        o = odict()
        msg = (
            'update() of ordered dict takes no keyword arguments to avoid '
            'an ordering trap.'
        )
        self.assertRaisesWithMessage(msg, o.update, TypeError, foo=1)

    def test_update(self):
        # Test update().
        o2 = odict()
        o2.update(data=((1, 1), (2, 2)))
        self.assertEqual(o2.items(), [(1, 1), (2, 2)])
        o2.update(data={3: 3})
        self.assertEqual(o2.items(), [(1, 1), (2, 2), (3, 3)])

    def test_first_and_last_key(self):
        # Test first and last key.
        o = odict([('a', 1), ('b', 2), ('c', 3), ('d', 4)])
        self.assertEqual(o.firstkey(), 'a')
        self.assertEqual(o.first_key, 'a')
        self.assertEqual(o.lastkey(), 'd')
        self.assertEqual(o.last_key, 'd')
        o = odict()
        msg = "'Ordered dictionary is empty'"
        self.assertRaisesWithMessage(msg, o.firstkey, KeyError)
        self.assertRaisesWithMessage(msg, lambda: o.first_key, KeyError)
        msg = "'Ordered dictionary is empty'"
        self.assertRaisesWithMessage(msg, o.lastkey, KeyError)
        self.assertRaisesWithMessage(msg, lambda: o.last_key, KeyError)

    def test_reverse_iteration(self):
        # Reverse iteration
        o = odict([('a', 1), ('b', 2), ('c', 3), ('d', 4)])
        self.assertEqual([x for x in o.riterkeys()], ['d', 'c', 'b', 'a'])
        self.assertEqual(o.rkeys(), ['d', 'c', 'b', 'a'])
        self.assertEqual([x for x in o.ritervalues()], [4, 3, 2, 1])
        self.assertEqual(o.rvalues(), [4, 3, 2, 1])
        self.assertEqual(
            [x for x in o.riteritems()],
            [('d', 4), ('c', 3), ('b', 2), ('a', 1)]
        )
        self.assertEqual(
            o.ritems(),
            [('d', 4), ('c', 3), ('b', 2), ('a', 1)]
        )

    def test_fromkeys(self):
        # From keys initialization::
        o = odict.fromkeys((1, 2, 3), 'x')
        self.assertEqual(o.items(), [(1, 'x'), (2, 'x'), (3, 'x')])

    def test_setdefault(self):
        # Test ``setdefault``.
        o = odict([(1, 'x')])
        self.assertEqual(o.setdefault(1, 9999), 'x')
        self.assertEqual(o.setdefault(4, 9999), 9999)

    def test_pop(self):
        # Test pop and popitem
        o = odict([(1, 'a'), (2, 'b')])
        self.assertRaisesWithMessage('3', o.pop, KeyError, 3)
        self.assertEqual(o.pop(3, 'foo'), 'foo')
        self.assertEqual(o.pop(2), 'b')
        self.assertEqual(o.popitem(), (1, 'a'))
        msg = "\"'popitem(): ordered dictionary is empty'\""
        self.assertRaisesWithMessage(msg, o.popitem, KeyError)

    def test_delete(self):
        # Removal from empty odict
        o = odict()

        def delete():
            del o['1']
        self.assertRaisesWithMessage("'1'", delete, KeyError)
        # Removal from odict with one element
        o['1'] = 1
        del o['1']
        self.assertEqual([o.lh, o.lt, o], [_nil, _nil, odict()])
        self.assertEqual(
            o._repr(),
            'odict low level repr lh,lt,data: nil, nil, {}'
        )
        # Remove first element of the odict sequence
        o = odict([('1', 1), ('2', 2), ('3', 3)])
        del o['1']
        self.assertEqual(
            [o.lh, o.lt, o],
            ['2', '3', odict([('2', 2), ('3', 3)])]
        )
        # Remove element in the middle of the odict sequence
        o = odict([('1', 1), ('2', 2), ('3', 3)])
        del o['2']
        self.assertEqual(
            [o.lh, o.lt, o],
            ['1', '3', odict([('1', 1), ('3', 3)])]
        )
        # Remove element at the end of the odict sequence
        o = odict([('1', 1), ('2', 2), ('3', 3)])
        del o['3']
        self.assertEqual(
            [o.lh, o.lt, o],
            ['1', '2', odict([('1', 1), ('2', 2)])],
        )

    def test_copy(self):
        # test copy function on odict
        o = odict([('a', 1), ('b', 2), ('c', 3), ('d', 4)])
        o_copy = o.copy()
        self.assertTrue(o_copy is not o)
        self.assertEqual(
            o_copy.items(),
            [('a', 1), ('b', 2), ('c', 3), ('d', 4)]
        )
        # test with copy module
        o = odict([('1', 1), ('2', 2), ('3', object())])
        # shallow copy
        o_copy = copy.copy(o)
        self.assertFalse(o_copy is o)
        self.assertTrue(o_copy['3'] is o['3'])
        # deep copy
        o_copy = copy.deepcopy(o)
        self.assertFalse(o_copy is o)
        self.assertFalse(o_copy['3'] is o['3'])

    def test_casting(self):
        # in python < 3.7 casting to dict will fail
        # Reason -> http://bugs.python.org/issue1615701
        # The __init__ function of dict checks wether arg is subclass of dict,
        # and ignores overwritten __getitem__ & co if so.
        # This was fixed and later reverted due to behavioural problems with
        # pickle.
        if sys.version_info < (3, 7):  # pragma: no cover
            self.assertEqual(dict(odict([(1, 1)])), {1: [_nil, 1, _nil]})
        else:  # pragma: no cover
            self.assertEqual(dict(odict([(1, 1)])), {1: 1})
        # The following ways for type conversion work
        self.assertEqual(dict(odict([(1, 1)]).items()), {1: 1})
        self.assertEqual(odict([(1, 1)]).as_dict(), {1: 1})

    def test_pickle(self):
        # dump and load odict pickle
        self.assertEqual(
            pickle.loads(pickle.dumps([odict([(1, 2)])])),
            [odict([(1, 2)])]
        )

    def test_sort(self):
        # basic sorting
        o = odict([('a', 1), ('c', 3), ('b', 2)])
        o.sort()
        self.assertEqual(o.items(), [('a', 1), ('b', 2), ('c', 3)])
        # A custom cmp function. Note that you get (key, value) tuples to
        # compare. Test with cmp function which sorting by key in reversed
        # order

        def mycmp(x, y):
            if x[0] > y[0]:
                return -1
            if x[0] < y[0]:
                return 1
        o = odict([('a', 1), ('c', 3), ('b', 2)])
        o.sort(cmp=mycmp)
        self.assertEqual(o.items(), [('c', 3), ('b', 2), ('a', 1)])
        # Test key and reverse kwargs
        o = odict([('a', 1), ('c', 3), ('b', 2)])
        o.sort(key=lambda x: x[0])
        self.assertEqual(o.items(), [('a', 1), ('b', 2), ('c', 3)])
        o.sort(key=lambda x: x[0], reverse=True)
        self.assertEqual(o.items(), [('c', 3), ('b', 2), ('a', 1)])

    def test_getattr_setattr_subclass(self):
        class Sub(odict):
            def __getattr__(self, name):
                try:
                    return self[name]
                except KeyError:
                    raise AttributeError(name)

            def __setattr__(self, name, value):
                self[name] = value

        sub = Sub()
        sub.title = 'foo'
        self.assertEqual(sub.keys(), ['title'])
        self.assertEqual(sub.title, 'foo')

    def test_bool_expessions(self):
        # Check boolean expressions.
        self.assertFalse(odict() and True or False)
        self.assertTrue(odict([('a', 1)]) and True or False)
        self.assertTrue(bool(odict([('a', 1)])))

    def test_alter_key(self):
        # Test alter_key function.
        o = odict((('1', 'a'), ('2', 'b'), ('3', 'c')))
        self.assertEqual(o.keys(), ['1', '2', '3'])
        o.alter_key('1', 'foo')
        self.assertEqual(o.keys(), ['foo', '2', '3'])
        self.assertTrue(o._dict_impl() is dict)
        dict_values = o._dict_impl().values(o)
        self.assertEqual(len(dict_values), 3)
        self.assertTrue(['foo', 'b', '3'] in dict_values)
        self.assertTrue(['2', 'c', _nil] in dict_values)
        self.assertTrue([_nil, 'a', '2'] in dict_values)
        self.assertEqual(o.values(), ['a', 'b', 'c'])
        self.assertEqual(o['foo'], 'a')
        self.assertEqual(o.lh, 'foo')
        o.alter_key('2', 'bar')
        self.assertEqual(o.keys(), ['foo', 'bar', '3'])
        dict_values = o._dict_impl().values(o)
        self.assertEqual(len(dict_values), 3)
        self.assertTrue(['bar', 'c', _nil] in dict_values)
        self.assertTrue(['foo', 'b', '3'] in dict_values)
        self.assertTrue([_nil, 'a', 'bar'] in dict_values)
        self.assertEqual(o.values(), ['a', 'b', 'c'])
        self.assertEqual(o['bar'], 'b')
        o.alter_key('3', 'baz')
        self.assertEqual(o.keys(), ['foo', 'bar', 'baz'])
        dict_values = o._dict_impl().values(o)
        self.assertEqual(len(dict_values), 3)
        self.assertTrue(['foo', 'b', 'baz'] in dict_values)
        self.assertTrue([_nil, 'a', 'bar'] in dict_values)
        self.assertTrue(['bar', 'c', _nil] in dict_values)
        self.assertEqual(o.values(), ['a', 'b', 'c'])
        self.assertEqual(o['baz'], 'c')
        self.assertEqual(o.lt, 'baz')

    def test_str(self):
        # Check __str__ function
        o = odict([('foo', 'a'), ('bar', 'b'), ('baz', 'c')])
        self.assertEqual(str(o), "{'foo': 'a', 'bar': 'b', 'baz': 'c'}")

    def test_repr(self):
        self.assertEqual(repr(odict()), 'odict()')
        self.assertEqual(repr(odict([('foo', 'a')])), "odict([('foo', 'a')])")

    def test_swap(self):
        # Test ``swap``
        o = odict([('0', 'a'), ('1', 'b'), ('2', 'c'), ('3', 'd'), ('4', 'e')])
        # Cannot swap same key
        with self.assertRaises(ValueError):
            o.swap('0', '0')
        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
        # Case first 2, a < b
        o.swap('0', '1')
        self.assertEqual(o.keys(), ['1', '0', '2', '3', '4'])
        self.assertEqual(o.values(), ['b', 'a', 'c', 'd', 'e'])
        # Case first 2, a > b
        o.swap('0', '1')
        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
        # Case last 2, a < b
        o.swap('3', '4')
        self.assertEqual(o.keys(), ['0', '1', '2', '4', '3'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'e', 'd'])
        # Case last 2, a > b
        o.swap('3', '4')
        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
        # Case neighbors, a < b
        o.swap('1', '2')
        self.assertEqual(o.keys(), ['0', '2', '1', '3', '4'])
        self.assertEqual(o.values(), ['a', 'c', 'b', 'd', 'e'])
        # Case neighbors, a > b
        o.swap('1', '2')
        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
        # Case non neighbors, one key first, a < b
        o.swap('0', '2')
        self.assertEqual(o.keys(), ['2', '1', '0', '3', '4'])
        self.assertEqual(o.values(), ['c', 'b', 'a', 'd', 'e'])
        # Case non neighbors, one key first, a > b
        o.swap('0', '2')
        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
        # Case non neighbors, one key last, a < b
        o.swap('2', '4')
        self.assertEqual(o.keys(), ['0', '1', '4', '3', '2'])
        self.assertEqual(o.values(), ['a', 'b', 'e', 'd', 'c'])
        # Case non neighbors, one key last, a > b
        o.swap('2', '4')
        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
        # Case non neighbors, a < b
        o.swap('1', '3')
        self.assertEqual(o.keys(), ['0', '3', '2', '1', '4'])
        self.assertEqual(o.values(), ['a', 'd', 'c', 'b', 'e'])
        # Case non neighbors, a > b
        o.swap('1', '3')
        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])

    def test_insertbefore(self):
        o = odict([('0', 'a')])
        with self.assertRaises(ValueError):
            o.insertbefore('0', '0', 'a')
        with self.assertRaises(KeyError):
            o.insertbefore('x', '1', 'b')
        o.insertbefore('0', '1', 'b')
        self.assertEqual(o.keys(), ['1', '0'])
        self.assertEqual(o.values(), ['b', 'a'])
        o.insertbefore('0', '2', 'c')
        self.assertEqual(o.keys(), ['1', '2', '0'])
        self.assertEqual(o.values(), ['b', 'c', 'a'])

    def test_insertafter(self):
        o = odict([('0', 'a')])
        with self.assertRaises(ValueError):
            o.insertafter('0', '0', 'a')
        with self.assertRaises(KeyError):
            o.insertafter('x', '1', 'b')
        o.insertafter('0', '1', 'b')
        self.assertEqual(o.keys(), ['0', '1'])
        self.assertEqual(o.values(), ['a', 'b'])
        o.insertafter('0', '2', 'c')
        self.assertEqual(o.keys(), ['0', '2', '1'])
        self.assertEqual(o.values(), ['a', 'c', 'b'])

    def test_insertfirst(self):
        o = odict()
        o.insertfirst('0', 'a')
        self.assertEqual(o.keys(), ['0'])
        self.assertEqual(o.values(), ['a'])
        o.insertfirst('1', 'b')
        self.assertEqual(o.keys(), ['1', '0'])
        self.assertEqual(o.values(), ['b', 'a'])

    def test_insertlast(self):
        o = odict()
        o.insertlast('0', 'a')
        self.assertEqual(o.keys(), ['0'])
        self.assertEqual(o.values(), ['a'])
        o.insertlast('1', 'b')
        self.assertEqual(o.keys(), ['0', '1'])
        self.assertEqual(o.values(), ['a', 'b'])

    def test_next_key(self):
        o = odict()
        with self.assertRaises(KeyError):
            o.next_key('x')
        o['x'] = 'x'
        with self.assertRaises(KeyError):
            o.next_key('x')
        o['y'] = 'y'
        self.assertEqual(o.next_key('x'), 'y')

    def test_prev_key(self):
        o = odict()
        with self.assertRaises(KeyError):
            o.prev_key('x')
        o['x'] = 'x'
        with self.assertRaises(KeyError):
            o.prev_key('x')
        o['y'] = 'y'
        self.assertEqual(o.prev_key('y'), 'x')


if __name__ == '__main__':
    unittest.main()  # pragma: no cover



# Python Software Foundation License
from __future__ import print_function
from collections import OrderedDict
from pyodict import odict
import time


CREATE_DELETE_ROW = '+----------------+---------------+'
RELATION_ROW = '+---------------------------+-----------+'

root = {}


def create(factory, _range):
    for i in range(_range):
        root[str(i)] = factory()


def delete(_range):
    for i in range(_range):
        del root[str(i)]


def result(factory, _range):
    start = time.clock()
    create(factory, _range)
    mid = time.clock()
    delete(_range)
    end = time.clock()
    print(
        '| Add    {0:7d} | {1:11.5f}ms |'.format(
            _range,
            (mid - start) * 1000,
        )
    )
    print(CREATE_DELETE_ROW)
    print(
        '| Delete {0:7d} | {1:11.5f}ms |'.format(
            _range,
            (end - mid) * 1000.0,
        )
    )
    print(CREATE_DELETE_ROW)
    return start, mid, end


def relation_row(action, _range, relation):
    print(
        '| {0:6s} {1:8d} objects |  1:{2:6.3f} |'.format(
            action,
            _range,
            relation,
        )
    )
    print(RELATION_ROW)


def head(value):
    print('')
    print(value)
    print('')


def run():
    head('adding and deleting builtin ``dict`` objects')
    print(CREATE_DELETE_ROW)
    dict_results = {
        1000: result(dict, 1000),
        10000: result(dict, 10000),
        100000: result(dict, 100000),
        1000000: result(dict, 1000000),
    }

    head('adding and deleting ``collection.OrderedDict`` objects')
    print(CREATE_DELETE_ROW)
    ordereddict_results = {
        1000: result(OrderedDict, 1000),
        10000: result(OrderedDict, 10000),
        100000: result(OrderedDict, 100000),
        1000000: result(OrderedDict, 1000000),
    }

    head('adding and deleting ``odict`` objects provided by this package')
    print(CREATE_DELETE_ROW)
    odict_results = {
        1000: result(odict, 1000),
        10000: result(odict, 10000),
        100000: result(odict, 100000),
        1000000: result(odict, 1000000),
    }

    head('relation ``dict : odict``')
    print(RELATION_ROW)
    for key, value in dict_results.items():
        dstart, dmid, dend = value
        ostart, omid, oend = odict_results[key]
        relation_create = (omid - ostart) / (dmid - dstart)
        relation_delete = (oend - omid) / (dend - dmid)
        relation_row('creating', key, relation_create)
        relation_row('deleting', key, relation_delete)

    head('relation ``OrderedDict : odict``')
    print(RELATION_ROW)
    for key, value in ordereddict_results.items():
        dstart, dmid, dend = value
        ostart, omid, oend = odict_results[key]
        relation_create = (omid - ostart) / (dmid - dstart)
        relation_delete = (oend - omid) / (dend - dmid)
        relation_row('creating', key, relation_create)
        relation_row('deleting', key, relation_delete)

if __name__ == '__main__':
    run()



# Python Software Foundation License
import copy
import functools
import sys


ITER_FUNC = 'iteritems' if sys.version_info[0] < 3 else 'items'


class _Nil(object):
    """Q: it feels like using the class with "is" and "is not" instead of
    "==" and "!=" should be faster.

    A: This would break implementations which use pickle for persisting.
    """

    def __repr__(self):
        return "nil"

    def __eq__(self, other):
        if (isinstance(other, _Nil)):
            return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if (isinstance(other, _Nil)):
            return False
        else:
            return NotImplemented

    def __hash__(self):
        return sys.maxsize


_nil = _Nil()


class _odict(object):
    """Ordered dict data structure, with O(1) complexity for dict operations
    that modify one element.

    Overwriting values doesn't change their original sequential order.
    """

    def _dict_impl(self):
        return None

    def _list_factory(self):
        return list

    def __init__(self, data=(), **kwds):
        """This doesn't accept keyword initialization as normal dicts to avoid
        a trap - inside a function or method the keyword args are accessible
        only as a dict, without a defined order, so their original order is
        lost.
        """
        if kwds:
            raise TypeError("__init__() of ordered dict takes no keyword "
                            "arguments to avoid an ordering trap.")
        dict_ = self._dict_impl()
        if dict_ is None:
            raise TypeError("No dict implementation class provided.")
        dict_.__init__(self)
        # If you give a normal dict, then the order of elements is undefined
        if hasattr(data, ITER_FUNC):
            for key, val in getattr(data, ITER_FUNC)():
                self[key] = val
        else:
            for key, val in data:
                self[key] = val

    # Double-linked list header
    @property
    def lh(self):
        dict_ = self._dict_impl()
        if not hasattr(self, '_lh'):
            dict_.__setattr__(self, '_lh', _nil)
        return dict_.__getattribute__(self, '_lh')

    @lh.setter
    def lh(self, val):
        self._dict_impl().__setattr__(self, '_lh', val)

    # Double-linked list tail
    @property
    def lt(self):
        dict_ = self._dict_impl()
        if not hasattr(self, '_lt'):
            dict_.__setattr__(self, '_lt', _nil)
        return dict_.__getattribute__(self, '_lt')

    @lt.setter
    def lt(self, val):
        self._dict_impl().__setattr__(self, '_lt', val)

    def __getitem__(self, key):
        return self._dict_impl().__getitem__(self, key)[1]

    def __setitem__(self, key, val):
        dict_ = self._dict_impl()
        try:
            dict_.__getitem__(self, key)[1] = val
        except KeyError:
            list_ = self._list_factory()
            lt = dict_.__getattribute__(self, 'lt')
            new = list_([lt, val, _nil])
            dict_.__setitem__(self, key, new)
            if lt == _nil:
                dict_.__setattr__(self, 'lh', key)
            else:
                dict_.__getitem__(self, lt)[2] = key
            dict_.__setattr__(self, 'lt', key)

    def __delitem__(self, key):
        dict_ = self._dict_impl()
        pred, _, succ = dict_.__getitem__(self, key)
        if pred == _nil:
            dict_.__setattr__(self, 'lh', succ)
        else:
            dict_.__getitem__(self, pred)[2] = succ
        if succ == _nil:
            dict_.__setattr__(self, 'lt', pred)
        else:
            dict_.__getitem__(self, succ)[0] = pred
        dict_.__delitem__(self, key)

    def __copy__(self):
        new = type(self)()
        for k, v in self.iteritems():
            new[k] = v
        new.__dict__.update(self.__dict__)
        return new

    def __deepcopy__(self, memo):
        new = type(self)()
        memo[id(self)] = new
        for k, v in self.iteritems():
            new[k] = copy.deepcopy(v, memo)
        for k, v in getattr(self.__dict__, ITER_FUNC)():
            setattr(new, k, copy.deepcopy(v, memo))
        return new

    def __contains__(self, key):
        try:
            self[key]
            return True
        except KeyError:
            return False

    def has_key(self, key):
        return key in self

    def __len__(self):
        return len(self.keys())

    def __str__(self):
        pairs = ("%r: %r" % (k, v) for k, v in getattr(self, ITER_FUNC)())
        return "{%s}" % ", ".join(pairs)

    def __repr__(self):
        if self:
            pairs = (
                "(%r, %r)" % (k, v) for k, v in getattr(self, ITER_FUNC)()
            )
            return "%s([%s])" % (self.__class__.__name__, ", ".join(pairs))
        else:
            return "%s()" % self.__class__.__name__

    def get(self, k, x=None):
        if k in self:
            return self._dict_impl().__getitem__(self, k)[1]
        else:
            return x

    def __iter__(self):
        dict_ = self._dict_impl()
        curr_key = dict_.__getattribute__(self, 'lh')
        while curr_key != _nil:
            yield curr_key
            curr_key = dict_.__getitem__(self, curr_key)[2]

    iterkeys = __iter__

    def keys(self):
        return list(self.iterkeys())

    def alter_key(self, old_key, new_key):
        dict_ = self._dict_impl()
        list_ = self._list_factory()
        val = dict_.__getitem__(self, old_key)
        dict_.__delitem__(self, old_key)
        if val[0] != _nil:
            prev = dict_.__getitem__(self, val[0])
            dict_.__setitem__(self, val[0], list_([prev[0], prev[1], new_key]))
        else:
            dict_.__setattr__(self, 'lh', new_key)
        if val[2] != _nil:
            next = dict_.__getitem__(self, val[2])
            dict_.__setitem__(self, val[2], list_([new_key, next[1], next[2]]))
        else:
            dict_.__setattr__(self, 'lt', new_key)
        dict_.__setitem__(self, new_key, val)

    def itervalues(self):
        dict_ = self._dict_impl()
        curr_key = dict_.__getattribute__(self, 'lh')
        while curr_key != _nil:
            _, val, curr_key = dict_.__getitem__(self, curr_key)
            yield val

    def values(self):
        return list(self.itervalues())

    def iteritems(self):
        dict_ = self._dict_impl()
        curr_key = dict_.__getattribute__(self, 'lh')
        while curr_key != _nil:
            _, val, next_key = dict_.__getitem__(self, curr_key)
            yield curr_key, val
            curr_key = next_key

    def items(self):
        return list(self.iteritems())

    def sort(self, cmp=None, key=None, reverse=False):
        items = [(k, v) for k, v in self.iteritems()]
        if cmp is not None:
            key = functools.cmp_to_key(cmp)
        if key is not None:
            items = sorted(items, key=key)
        else:
            items = sorted(items, key=lambda x: x[1])
        if reverse:
            items.reverse()
        self.clear()
        self.__init__(items)

    def clear(self):
        dict_ = self._dict_impl()
        dict_.clear(self)
        dict_.__setattr__(self, 'lh', _nil)
        dict_.__setattr__(self, 'lt', _nil)

    def copy(self):
        return self.__class__(self)

    def update(self, data=(), **kwds):
        if kwds:
            raise TypeError(
                "update() of ordered dict takes no keyword arguments to avoid "
                "an ordering trap."
            )
        if hasattr(data, ITER_FUNC):
            data = getattr(data, ITER_FUNC)()
        for key, val in data:
            self[key] = val

    def setdefault(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            self[key] = default
            return default

    def pop(self, key, default=_nil):
        try:
            val = self[key]
            del self[key]
            return val
        except KeyError:
            if default == _nil:
                raise
            return default

    def popitem(self):
        try:
            dict_ = self._dict_impl()
            key = dict_.__getattribute__(self, 'lt')
            return key, self.pop(key)
        except KeyError:
            raise KeyError("'popitem(): ordered dictionary is empty'")

    def riterkeys(self):
        """To iterate on keys in reversed order.
        """
        dict_ = self._dict_impl()
        curr_key = dict_.__getattribute__(self, 'lt')
        while curr_key != _nil:
            yield curr_key
            curr_key = dict_.__getitem__(self, curr_key)[0]

    __reversed__ = riterkeys

    def rkeys(self):
        """List of the keys in reversed order.
        """
        return list(self.riterkeys())

    def ritervalues(self):
        """To iterate on values in reversed order.
        """
        dict_ = self._dict_impl()
        curr_key = dict_.__getattribute__(self, 'lt')
        while curr_key != _nil:
            curr_key, val, _ = dict_.__getitem__(self, curr_key)
            yield val

    def rvalues(self):
        """List of the values in reversed order.
        """
        return list(self.ritervalues())

    def riteritems(self):
        """To iterate on (key, value) in reversed order.
        """
        dict_ = self._dict_impl()
        curr_key = dict_.__getattribute__(self, 'lt')
        while curr_key != _nil:
            pred_key, val, _ = dict_.__getitem__(self, curr_key)
            yield curr_key, val
            curr_key = pred_key

    def ritems(self):
        """List of the (key, value) in reversed order.
        """
        return list(self.riteritems())

    def firstkey(self):
        if self:
            return self._dict_impl().__getattribute__(self, 'lh')
        else:
            raise KeyError('Ordered dictionary is empty')

    @property
    def first_key(self):
        return self.firstkey()

    def lastkey(self):
        if self:
            return self._dict_impl().__getattribute__(self, 'lt')
        else:
            raise KeyError('Ordered dictionary is empty')

    @property
    def last_key(self):
        return self.lastkey()

    def as_dict(self):
        return self._dict_impl()(self.iteritems())

    def _repr(self):
        """_repr(): low level repr of the whole data contained in the odict.
        Useful for debugging.
        """
        dict_ = self._dict_impl()
        form = "odict low level repr lh,lt,data: %r, %r, %s"
        return form % (
            dict_.__getattribute__(self, 'lh'),
            dict_.__getattribute__(self, 'lt'),
            dict_.__repr__(self)
        )

    def swap(self, a, b):
        if a == b:
            raise ValueError('Swap keys are equal')
        dict_ = self._dict_impl()
        list_ = self._list_factory()
        orgin_a = dict_.__getitem__(self, a)
        orgin_b = dict_.__getitem__(self, b)
        new_a = list_([orgin_b[0], orgin_a[1], orgin_b[2]])
        new_b = list_([orgin_a[0], orgin_b[1], orgin_a[2]])
        if new_a[0] == a:
            new_a[0] = b
            new_b[2] = a
        if new_b[0] == b:
            new_b[0] = a
            new_a[2] = b
        if new_a[0] != _nil:
            dict_.__getitem__(self, new_a[0])[2] = a
        if new_a[2] != _nil:
            dict_.__getitem__(self, new_a[2])[0] = a
        if new_b[0] != _nil:
            dict_.__getitem__(self, new_b[0])[2] = b
        if new_b[2] != _nil:
            dict_.__getitem__(self, new_b[2])[0] = b
        dict_.__setitem__(self, a, new_a)
        dict_.__setitem__(self, b, new_b)
        if new_a[0] == _nil:
            dict_.__setattr__(self, 'lh', a)
        if new_a[2] == _nil:
            dict_.__setattr__(self, 'lt', a)
        if new_b[0] == _nil:
            dict_.__setattr__(self, 'lh', b)
        if new_b[2] == _nil:
            dict_.__setattr__(self, 'lt', b)

    def insertbefore(self, ref, key, value):
        if ref == key:
            raise ValueError('Reference key and new key are equal')
        try:
            index = self.keys().index(ref)
        except ValueError:
            raise KeyError('Reference key \'{}\' not found'.format(ref))
        prevkey = prevval = None
        dict_ = self._dict_impl()
        list_ = self._list_factory()
        if index > 0:
            prevkey = self.keys()[index - 1]
            prevval = dict_.__getitem__(self, prevkey)
        if prevval is not None:
            dict_.__getitem__(self, prevkey)[2] = key
            newval = list_([prevkey, value, ref])
        else:
            dict_.__setattr__(self, 'lh', key)
            newval = list_([_nil, value, ref])
        dict_.__getitem__(self, ref)[0] = key
        dict_.__setitem__(self, key, newval)

    def insertafter(self, ref, key, value):
        if ref == key:
            raise ValueError('Reference key and new key are equal')
        try:
            index = self.keys().index(ref)
        except ValueError:
            raise KeyError('Reference key \'{}\' not found'.format(ref))
        nextkey = nextval = None
        keys = self.keys()
        dict_ = self._dict_impl()
        list_ = self._list_factory()
        if index < len(keys) - 1:
            nextkey = keys[index + 1]
            nextval = dict_.__getitem__(self, nextkey)
        if nextval is not None:
            dict_.__getitem__(self, nextkey)[0] = key
            newval = list_([ref, value, nextkey])
        else:
            dict_.__setattr__(self, 'lt', key)
            newval = list_([ref, value, _nil])
        dict_.__getitem__(self, ref)[2] = key
        dict_.__setitem__(self, key, newval)

    def insertfirst(self, key, value):
        keys = self.keys()
        if not keys:
            self[key] = value
            return
        self.insertbefore(keys[0], key, value)

    def insertlast(self, key, value):
        keys = self.keys()
        if not keys:
            self[key] = value
            return
        self.insertafter(keys[-1], key, value)

    def next_key(self, key):
        dict_ = self._dict_impl()
        curr = dict_.__getitem__(self, key)
        if curr[2] == _nil:
            raise KeyError('No next key')
        return curr[2]

    def prev_key(self, key):
        dict_ = self._dict_impl()
        curr = dict_.__getitem__(self, key)
        if curr[0] == _nil:
            raise KeyError('No previous key')
        return curr[0]


class odict(_odict, dict):

    def _dict_impl(self):
        return dict



# Python Software Foundation License
from setuptools import find_packages
from setuptools import setup
import os


def read_file(name):
    with open(os.path.join(os.path.dirname(__file__), name)) as f:
        return f.read()


version = '1.8.1'
shortdesc = 'Ordered Dictionary.'
longdesc = '\n\n'.join([read_file(name) for name in [
    'README.rst',
    'CHANGES.rst',
    'LICENSE.rst'
]])


setup(
    name='odict',
    version=version,
    description=shortdesc,
    long_description=longdesc,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Python Software Foundation License',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    keywords='odict dict ordered dictionary mapping collection tree',
    author='Node Contributors',
    author_email='dev@conestack.org',
    url='https://github.com/conestack/odict',
    license='Python Software Foundation License',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=[],
    include_package_data=True,
    zip_safe=True,
    install_requires=['setuptools'],
    test_suite='odict.tests'
)
